@@ -229,6 +229,11 @@ def load_enrichjson_defaults(target, usecase=True):
     else:
         enrichjsons = glob2.glob(target + "/*enrich.json")
 
+        # Initialize
+        defaults['org_name'] = os.path.basename(os.path.abspath(target))
+        defaults['usecase_name'] = "Unknown"
+
+
     for enrichjson in enrichjsons:
       try:
         j = json.load(open(enrichjson))
@@ -349,16 +354,18 @@ def prepare_setup(component, target):
     conf = read_conf()
 
     basename = os.path.basename(target)
+    repo = conf.get('repository',{})
+    org = conf.get('org',{})
     params = {
         'component': component,
         'basename': basename,
         'name': basename,
         'version': '0.1',
         'description': basename + " package",
-        'author': conf['repository']['author'],
-        'email': conf['repository']['email'],
-        'url': conf['repository']['giturl'],
-        'customer': conf['org']['name']
+        'author': repo.get('author',''),
+        'email': repo.get('email',''),
+        'url': repo.get('giturl',''),
+        'customer': org.get('name','')
     }
 
 
@@ -661,8 +668,8 @@ def prepare_pipelineconf(path, templatepath):
         'realtime_or_batch': 'batch'
     }
     context = {
-        'customer': conf['org']['customer'],
-        'division': conf['org']['name'],
+        'customer': conf.get('org',{}).get('customer',''),
+        'division': conf.get('org',{}).get('name','')
     }
     variables = get_variables(lib_template)
     for v in variables:
@@ -2047,7 +2054,7 @@ def testdata_transform_list(pkgdir, capture, context):
     datasets = transform.testdata['datasets']
     available = datasets.get('available', [])
     for i, d in enumerate(available):
-        print("[{}] {}".format(i, d))
+        print("[{:2}] {}".format(i, d))
 
 def testdata_spec_list(specfile, capture, context):
     """
@@ -2070,7 +2077,7 @@ def testdata_spec_list(specfile, capture, context):
         return
 
     for i, d in enumerate(registry.list()):
-        print("[{}] {}".format(i, d))
+        print("[{:2}] {}".format(i, d))
 
 def testdata_transform_show(pkgdir, capture, context, name):
     """
@@ -2138,6 +2145,82 @@ def testdata_transform_show(pkgdir, capture, context, name):
     for example in examples:
         print("   :", example['timestamp'], "=>", example['name'])
 
+def testdata_spec_show_file(config, registry, dataset):
+    """
+    Show commands involving a file
+    """
+
+    extra = registry.params
+
+    print("[Targets] Available ", end='')
+    print("(files)" if dataset.isfile else "(dirs)")
+    print("------")
+    for p in dataset.get_paths():
+        name = p['name']
+        path = p['path']
+        nature = p['nature']
+        resolved = config.get_file(path,
+                                   extra=extra,
+                                   abspath=False)
+        print("  [{}] (type: {}) {}".format(name, nature, resolved))
+
+    print("------")
+    print("[match] Pattern:", dataset.match['pattern'])
+    print("------")
+    dt = datetime.now().replace(hour=0, minute=0,
+                                second=0,microsecond=0)
+    examples = dataset.generate(end=dt,
+                                start=dt + timedelta(days=-7))
+
+    for example in examples:
+        print("   :", example['timestamp'], "=>", example['name'])
+
+    print("------")
+    print("Supported Commands")
+    print("------")
+    targets = defaultdict(list)
+    for p in dataset.get_paths():
+        name = p['name']
+        nature = p['nature']
+        targets[nature].append(name)
+
+    for c in registry.get_commands(source_type="File"):
+        name   = c['name']
+        desc   = c['description']
+        args   = c['args']
+
+        found = True
+        for arg in args:
+            values = arg.find(dataset)
+            if ((values is None) or
+                (values in ['']) or
+                ((isinstance(values, list)) and (len(values) == 0))):
+                found = False
+        if not found:
+            continue
+
+        print("\n[{}] {}".format(name, desc))
+        example = ""
+        for arg in args:
+            values = arg.find(dataset)
+
+            if not isinstance(values, list):
+                values = [str(values)]
+            else:
+                values = [str(v) for v in values]
+            print("   {:10s}:".format(arg.name),
+                  ",".join(values))
+
+            example += "{}='{}' ".format(arg.name, values[0])
+        print("\n   Example: ...",name, example)
+
+def testdata_spec_show_database(config, registry, dataset):
+    """
+    Show commands involving a database
+    """
+    for c in registry.get_commands(source_type="DBTable"):
+        pass
+
 def testdata_spec_show(specfile, capture, context, name):
     """
     Show details of a dataset
@@ -2179,59 +2262,12 @@ def testdata_spec_show(specfile, capture, context, name):
 
     print("Description:", dataset.description)
     print("------")
-    print("[Targets] Available " + "(files)" if dataset.isfile else "(dirs)")
-    print("------")
-    for p in dataset.get_paths():
-        name = p['name']
-        path = p['path']
-        nature = p['nature']
-        resolved = config.get_file(path,
-                                   extra=extra,
-                                   abspath=False)
-        print("  [{}] (type: {}) {}".format(name, nature, resolved))
-
-    print("------")
-    print("[match] Pattern:", dataset.match['pattern'])
-    print("------")
-    dt = datetime.now().replace(hour=0, minute=0,
-                                second=0,microsecond=0)
-    examples = dataset.generate(end=dt,
-                                start=dt + timedelta(days=-7))
-
-    for example in examples:
-        print("   :", example['timestamp'], "=>", example['name'])
-
-    print("------")
-    print("[Download] Possible Commands")
-    print("------")
-    targets = defaultdict(list)
-    for p in dataset.get_paths():
-        name = p['name']
-        nature = p['nature']
-        targets[nature].append(name)
-
-    for c in registry.commands:
-        name = c['name']
-        combination = c['valid']
-        print(name, "=>", combination)
-        if ((combination[0] not in targets) or
-            (combination[1] not in targets)):
-            print("  ", redbg("No valid combinations"))
-            continue
-
-        found = False
-        for name1 in targets[combination[0]]:
-            for name2 in targets[combination[1]]:
-                if name1 == name2:
-                    continue
-                print("   ",greenbg("{} => {}".format(name1, name2)))
-                found = True
-        if not found:
-            print("  ", redbg("No valid combinations"))
+    if dataset._type == 'File':
+        testdata_spec_show_file(config, registry, dataset)
 
 
-def testdata_transform_download(pkgdir, capture, context,
-                                name, target, start, end):
+def testdata_transform_action(pkgdir, capture, context,
+                              name, target, start, end):
     """
     Show details of a dataset
 
@@ -2306,8 +2342,8 @@ def testdata_transform_download(pkgdir, capture, context,
         })
         print(command % params)
 
-def testdata_spec_download(specfile, capture, context, name,
-                           command, src, dst, start, end):
+def testdata_spec_action(specfile, capture, context, name,
+                         command, **kwargs):
     """
     Show details of a dataset
 
@@ -2351,60 +2387,51 @@ def testdata_spec_download(specfile, capture, context, name,
     print("Description:", dataset.description)
     print("------")
 
-    # Get more details of all
-    try:
-        src = dataset.get_path_by_name(src, full=True)
-    except:
-        print(redbg("Source {} could not be found".format(src)))
-        return
-
-    try:
-        dst = dataset.get_path_by_name(dst, full=True)
-    except:
-        print(redbg("Dst {} could not be found".format(dst)))
-        return
-
     try:
         command = registry.get_command(command)
     except:
         print(redbg("Command {} could not be found".format(command)))
         return
 
-    if src['nature'] != command['valid'][0]:
-        print(redbg("src nature found {} expected {} ".format(src['nature'],
-                                                             command['valid'][0])))
-        return
+    print("Command")
+    print("   ", command['command'])
+    print("------")
 
-    if dst['nature'] != command['valid'][1]:
-        print(redbg("Dst nature found {} expected {}".format(dst['nature'],
-                                                             command['valid'][1])))
-        return
+    resolve = {}
+    args = command['args']
+    for arg in args:
+        name = arg.name
+        if arg.name not in kwargs:
+            raise Exception("Missing argument '{}' ({})".format(arg.name, arg.description))
 
+        value = kwargs[arg.name]
+        if not arg.validate(dataset,value):
+            raise Exception("Invalid argument '{}' ({}) has value '{}'".format(arg.name, arg.description, value))
 
-    src['path'] = config.get_file(src['path'],
-                                  extra=registry.params,
-                                  abspath=False)
-    dst['path'] = config.get_file(dst['path'],
-                                  extra=registry.params,
-                                  abspath=False)
+        try:
+            resolve[arg.name] = arg.resolve(dataset, value,resolve=registry.params)
+        except:
+            raise Exception("Invalid argument '{}' ({}) has value '{}'".format(arg.name, arg.description, value))
 
+    if (('start' not in resolve) or
+        ('end' not in resolve)):
+        raise Exception("'start' and 'end' datetime values are required")
 
-
-    try:
-        start = dateparser.parse(start) # naive datetime
-        end = dateparser.parse(end)
-    except:
-        print(redbg("Invalid start or end date"))
-        return
+    print("Args")
+    print("------")
+    for a in args:
+        print("   ", a.name, ":", resolve[a.name])
+    print("")
 
     print("[commands] Please run these after checking")
-    combinations = dataset.generate(end=end,start=start)
+    combinations = dataset.generate(end=resolve['end'],start=resolve['start'])
     for c in combinations:
         params = copy.copy(registry.params)
-        params['source'] = src['path']
-        params['destination'] = dst['path']
-        params['subdir'] = c['name']
-        params['slash'] = "" if dataset.isfile else "/"
+        params['source']      = resolve['src']
+        params['destination'] = resolve['dst']
+        params['subdir']      = c['name']
+        params['slash']       = "" if dataset.isfile else "/"
+
         try:
             print(command['command'] % params)
         except Exception as e:
@@ -2425,15 +2452,15 @@ def testdata_action(action, *args, **kwargs):
             testdata_transform_list(*args, **kwargs)
         elif action == 'show':
             testdata_transform_show(*args, **kwargs)
-        elif action == 'download':
-            testdata_transform_download(*args, **kwargs)
+        elif action == 'action':
+            testdata_transform_action(*args, **kwargs)
     else:
         if action == 'list':
             testdata_spec_list(*args, **kwargs)
         elif action == 'show':
             testdata_spec_show(*args, **kwargs)
-        elif action == 'download':
-            testdata_spec_download(*args, **kwargs)
+        elif action == 'action':
+            testdata_spec_action(*args, **kwargs)
 
 ###############################################
 # Main test
